#!/usr/bin/python
# @author: zhaofeng-shu33

import ace_internal
import numpy as np

def ace_cream(x, y, wt = None, delrsq = 0.01, ns = 1, cat = None):
    ''' 
    Uses the alternating conditional expectations algorithm
    to find the transformations of y and x that maximise the 
    proportion of variation in y explained by x.

    Parameters
    ----------
    x : array_like
        a matrix containing the independent variables.
        each row is an observation of data.
    y : array_like
        a vector containing the response variable.
    wt : array_like or None, optional
        an optional vector of weights (the default is equally weighted).
    delrsq : float, optional
        termination threshold (the default is 0.01). iteration stops when rsq changes 
        less than delrsq in 3 consecutive iterations
    ns : int, optional
        number of eigensolutions (sets of transformations, the default is 1).
    cat : list
        an optional integer vector specifying which variables assume categorical values. 
        nonnegative values in cat refer to columns of the x matrix and -1 refers to the response variable.
    Returns
    -------
    tx : array_like
        the transformed x values.
    ty : array_like
        the transformed y values.

    '''
    if wt is None:
        wt = np.ones(x.shape[0])
    if(len(x.shape) == 1): # if x is rank-1 matrix
        x_internal = x.reshape([x.shape[0],1])
    else:
        x_internal = x
    x_row = x_internal.shape[0]
    x_col = x_internal.shape[1]
    iy = x_col + 1 # len(col(x)) + 1
    l = np.ones(iy,'i') # flags for each variable
    if cat is not None:
        for i in cat:
            if (i  < -1 or i >= x_col):
                raise ValueError("bad cat= specification")
            if (i == -1):
                nncol = x_col
            else:
                nncol = i
            if (l[nncol] != 5 and l[nncol] != 1):
              raise ValueError("conflicting transformation specifications")
            l[nncol] = 5
    m = np.zeros([x_row, iy],'i')
    z = np.zeros([x_row, 12], 'd')
    p = x_internal.shape[1]
    tx, ty, rsq, ierr = ace_internal.mace(x = x_internal.T, y = y, w = wt, l = l, delrsq = delrsq, m = m, z= z, ns = ns, p = p, n = x_row)
    if not(ierr == 0): # number dimension overflow
        raise ValueError("number dimension overflow")
    if(ns == 1):
        return (tx[:,:,0], ty)
    else:
        return (tx, ty)

def f_mapping(x, tx, x_candidate):
    '''
    given x and tx, find f(x) = tx and return 
    t_x_candidate = f(x_candidate)
    only used for discrete random variable X

    Parameters
    ----------
    x : array-like or list
    tx : array-like
         transformated feature of x, same dimension with x
    x_candidate : array-like or list

    Returns
    -------
    t_x_candidate : array-like

    '''
    f = {}
    feature_vector_dim = tx.shape[1]
    for i in range(len(x)):
        f[x[i]] = tx[i,:]
    t_x_candidate = np.zeros([len(x_candidate), feature_vector_dim])

    for i in range(len(x_candidate)):
        if f.get(x_candidate[i]) is None:
            t_x_candidate[i,:] = x[0,:]
        else:
            t_x_candidate[i,:] = f[x_candidate[i]]
    return t_x_candidate

if __name__ == '__main__':
    TWOPI = 2 * np.pi
    x = np.random.uniform(0,TWOPI,200)
    y = np.exp(np.sin(x)+np.random.normal(size = 200)/2)
    tx, ty = ace_cream(x,y)
    



import os
from distutils.sysconfig import get_python_lib
# For gfortran+msvc combination, extra shared libraries may exist (stored by numpy.distutils)
if os.name == 'nt':
    HAS_DLL = False
    extra_dll_dir = os.path.join(get_python_lib(), 'ace_cream', '.libs')
    if os.path.isdir(extra_dll_dir):
        HAS_DLL = True
    else:
        extra_dll_dir = os.path.join(get_python_lib(), 'ace_internal', '.libs')
        if os.path.isdir(extra_dll_dir):
            HAS_DLL = True
    if(HAS_DLL):
        os.environ['PATH'] += os.pathsep + extra_dll_dir
    else:
        find_ace_internal_pyd = False
        for i in os.listdir(get_python_lib()):
            if(i.find('ace_internal')>=0 and i.find('pyd')>=0): 
                os.environ['PATH'] += get_python_lib()
                find_ace_internal_pyd = True
                break
        if not(find_ace_internal_pyd):
            raise ImportError('directory %s/ace_cream/.libs or %s/ace_internal/.libs not found'%(get_python_lib(),get_python_lib()))
from .ace_cream import ace_cream
from .ace_cream import f_mapping


#!/usr/bin/python

from numpy.distutils.core import Extension, setup
with open("README.rst") as fh:
    long_description = fh.read()
ext_internal = Extension(name = 'ace_internal', sources = ['ace.pyf',
                'ace.f', 'avas.f', 'rlsmo.f'
                ])
ext = Extension(name = 'ace_cream.ace_cream', sources = 'ace_cream.py')                
if __name__ == '__main__':
    setup(name = 'ace_cream',
          version = '0.4.post6',
          description = 'Alternating Conditional\
 Expectation Algorithm',
          author = 'zhaofeng-shu33',
          author_email = '616545598@qq.com',
          url = 'https://github.com/zhaofeng-shu33/ace_cream',
          maintainer = 'zhaofeng-shu33',
          maintainer_email = '616545598@qq.com',
          long_description = long_description,  
          install_requires = ['numpy'],
          ext_modules = [ext_internal],
          license = 'Apache License Version 2.0',
          packages = ['ace_cream'],
          classifiers = (
              "Development Status :: 4 - Beta",
              "Programming Language :: Fortran",
              "Programming Language :: Python :: 3.6",
              "Programming Language :: Python :: 3.5",
              "Programming Language :: Python :: 2.7",
              "Operating System :: OS Independent",
          ),
          )
